<?php

namespace Drupal\disable_language\EventSubscriber;

// This is the interface we are going to implement.
use Drupal\Core\Session\AccountProxy;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
// This class contains the event we want to subscribe to.
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\Core\Url;
use Drupal\disable_language\DisableLanguageManager;

/**
 * Subscribe to KernelEvents::REQUEST events.
 */
class DisabledLanguagesEventSubscriber implements EventSubscriberInterface {

  /**
   * Contains disable_language.disable_language_manager service.
   *
   * @var \Drupal\disable_language\DisableLanguageManager
   */
  protected $disableLanguageManager;

  /**
   * Contains current_user service.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * DisabledLanguagesEventSubscriber constructor.
   *
   * @param \Drupal\disable_language\DisableLanguageManager $disableLanguageManager
   *   Class DisableLanguageManager.
   * @param \Drupal\Core\Session\AccountProxy $currentUser
   *   A proxied implementation of AccountInterface.
   */
  public function __construct(DisableLanguageManager $disableLanguageManager, AccountProxy $currentUser) {
    $this->currentUser = $currentUser;
    $this->disableLanguageManager = $disableLanguageManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // On a normal request.
    $events[KernelEvents::REQUEST][] = ['checkForDisabledLanguageAndRedirect'];
    // On an access denied request.
    $events[KernelEvents::EXCEPTION][] = [
      'checkForDisabledLanguageAndRedirect',
      0,
    ];
    return $events;
  }

  /**
   * {@inheritdoc}
   */
  public function checkForDisabledLanguageAndRedirect(GetResponseEvent $event) {
    // Do not redirect if this is a file.
    $params = $event->getRequest()->attributes->all();
    if (isset($params['scheme']) && in_array($params['scheme'], ['public', 'private'])) {
      return;
    }
    // Redirect if the user doesn't have access to view disabled languages.
    elseif (!$this->currentUser->hasPermission('view disabled languages')) {
      if ($this->disableLanguageManager->isCurrentLanguageDisabled()) {
        // Get the configured redirect language if there is one.
        $redirect_language = $this->disableLanguageManager->getFallbackLanguage();
        if ($redirect_language) {
          $language = \Drupal::languageManager()
            ->getLanguage($redirect_language);
        }
        else {
          $language = $this->disableLanguageManager->getFirstEnabledLanguage();
        }
        if (isset($language)) {
          // Redirect to the homepage in the language passed along.
          $url = Url::fromRoute('<front>', [], ['language' => $language]);
          $response = new RedirectResponse($url->toString(), '307');
          $event->setResponse($response);
        }
      }
    }
  }

}
